'Mini-Datenlogger erweiterte Version
'
'Strommessung ber externen Verstrker
'Hhenmessung
'mit Telemetrie (Format Dietrich Meissner)



'#################################################################################################################################
'#
'#  MiniLogger.bas          Copyright Chris Benndorf/Thomas Marx
'#
'#  Betriebsprogramm fuer den in RC-Line diskutierten MiniLogger
'#  http://www.ganzfix.de/MiniLogger
'#
'#################################################################################################################################



'#################################################################################################################################
'#
'#  Die folgenden Konstanten dienen zur Anpassung an die verwendete Hardware
'#
'#################################################################################################################################

Const Version = 20                                          'Versionsnummer
Const Eeprom_adress_w = &HA0                                'Basisadresse des verwendeten EEPROMs 24C64A = &HA0 / 24C256 = &HA0
Const Eeprom_adress_r = Eeprom_adress_w + 1                 'Nicht aendern!

Const Use_aci_for_rpm = 1                                   'Drehzahl ueber Comperator Interrupt(1)
Const Use_int0_for_rpm = 0                                  'Drehzahl ueber INT0 Interrupt (1)
Const Use_positiv_ppm = 1                                   'Positives = 1 (z.B. MPX) oder negatives = 0 (z.B. Uralt-Robbe) PPM

'#################################################################################################################################
'#
'#  Compiler-Anweisungen
'#
'#################################################################################################################################

'$prog , &HF1 , ,                                            'Hardware-Einstellungen fuer internen 1 MHz Taktgeber
$crystal = 1000000                                          '1 MHz, Damit auch die Software davon weiss
$regfile = "at26def.dat"                                    'Wir benutzen einen Tiny26
$noramclear

'#################################################################################################################################
'#
'#  Variablen
'#
'#################################################################################################################################

Dim Eeprom_size As Word                                     '
Dim Measured_bytes As Word                                  '
Dim Last_icadr As Word                                      '
Dim Minrx As Byte                                           '
Dim Measure_freq As Byte                                    '

Dim Wtimer0 As Word At &H70                                 '16Bit-Timer
Dim Wtimer0lo As Byte At &H70 Overlay                       '
Dim Wtimer0hi As Byte At &H71 Overlay                       '
Dim Wcompare As Word At &H72                                '16Bit-Comparator
Dim Wcomparelo As Byte At &H72 Overlay                      '
Dim Wcomparehi As Byte At &H73 Overlay                      '
Dim Icadr As Word At &H74                                   'Adresse im I2C-EEProm
Dim Icadrlo As Byte At &H74 Overlay                         '
Dim Icadrhi As Byte At &H75 Overlay                         '
Dim Icadrsave As Word At &H76                               '
Dim Collected As Word At &H78                               'Anzahl gespeicherter Datenstze im akt. Block
Dim Collectedlo As Word At &H78 Overlay                     '
Dim Collectedhi As Word At &H79 Overlay                     '
Dim Collectedptr As Word At &H7A                            'Zeiger auf Lnge des akt. Datenblock
Dim Wtemp0 As Word At &H7C                                  '16Bit temporr
Dim Wtemp0lo As Byte At &H7C Overlay                        '
Dim Wtemp0hi As Byte At &H7D Overlay                        '
Dim Wtemp1 As Word At &H7E                                  '16Bit temporr
Dim Btemp As Byte                                           '8Bit temporr

Dim Sendbuffer(16) As Byte At &H90                          'Sendepuffer
Dim Y As Word At &H90 Overlay                               'dummy y
Dim T As Word At &H92 Overlay                               'Temperatur
Dim R As Word At &H94 Overlay                               'RC
Dim N As Word At &H96 Overlay                               'Drehzahl
Dim U As Word At &H98 Overlay                               'Spannung
Dim X As Word At &H9A Overlay                               'dummy x (Strom integral)
Dim I As Word At &H9C Overlay                               'Strom
Dim H As Word At &H9E Overlay                               'Hhe

'#################################################################################################################################
'#
'#  Hardware initialisieren
'#
'#################################################################################################################################

Ddra = &B00000000                                           'Ddra Input
Porta = &B00000000                                          'keine pull-up

Ddrb = &B00011010                                           'Ddrb.0 Input SDA
                                                            'DDrb.1 Output LED + Seriell
                                                            'DDrb.2 Input SCL
                                                            'DDrb.3 Output LED2
                                                            'DDrb.4 Output LED3
                                                            'DDrb.5 Input RX_In
                                                            'DDrb.6 Input Drehzahl (Chris)
                                                            'DDrb.7 Input (Reset)

Portb = &B01111000                                          'Portb.0 SDA
                                                            'Portb.1 LED / seriell
                                                            'Portb.2 SCL
                                                            'Portb.3
                                                            'Portb.4
                                                            'Portb.5 pull-up fr RX_In
                                                            'Portb.6 pull_up fr Drehzahl (Chris)
                                                            'Portb.7

Config Sda = Portb.0                                        'Ports fuer die I2C-Schnittstelle
Config Scl = Portb.2
'I2cinit                                                     'wird mit DDRB= und PORTB= miterledigt
                                                             'Reset Portb.0 (Sda)
                                                             'Reset Portb.2 (Scl)
                                                             'Reset Ddrb.0 (Sda)
                                                             'Reset DDrb.2 (Scl)

Open "comb.1:9600,8,n,1,inverted" For Output As #1          'Set Ddrb.1
                                                              'Reset Portb.1

'#################################################################################################################################
'#
'#  Interrupt-Quellen
'#
'#################################################################################################################################

Config Adc = Single , Prescaler = Auto                      'KEINE Angabe von Reference = Internal
Start Adc
On Adc _adc Nosave

On Ovf0 _ovf0 Nosave

#if Use_aci_for_rpm = 1                                     'Thomas' RPM
  Config Aci = On , Compare = Off , Trigger = Falling
  On Aci _aci Nosave
#endif
#if Use_int0_for_rpm = 1                                    'Chris' RPM
  Config Int0 = Rising
  On Int0 _aci Nosave
#endif

'#################################################################################################################################
'#
'#  Hauptprogramm
'#
'#################################################################################################################################

Wait 3                                                      'Warten bis alle Servos etc. fertig sind

Readeeprom Eeprom_size , 1                                  'Byte1+2
Readeeprom Measured_bytes , 3                               'Byte3
Readeeprom Last_icadr , 4                                   'Byte4+5
Readeeprom Minrx , 6                                        'Byte6
Readeeprom Measure_freq , 7                                 'Byte7

Icadr = 0                                                   'Header ins I2C-EEProm schreiben
Wtemp0hi = Version
Wtemp0lo = Measured_bytes
Gosub Write_eeprom                                          'Version und Measured-Bytes speichern
Wtemp0 = High(eeprom_size)
Gosub Write_eeprom                                          'eeprom_size speichern

Reset Portb.3                                               'LED2 an
Gosub Getrx0                                                'RC-Kanal lesen
If Wtemp0lo > Minrx Then                                    'RC-Kanal auf high?
  Icadr = 4
  Wtemp0 = &HFFFF
  Gosub Write_eeprom
End If
Do                                                          'warten bis RC-Kanal auf low ist
  Gosub Getrx0
Loop Until Wtemp0lo < Minrx

Icadr = 4
Gosub Read_eeprom
If Wtemp0 = &HFFFF Then                                     'wenn in Byte 4&5 &HFFFF steht ist der EEProm neu bzw. gelscht
  Collectedptr = 6                                          'Schreib-Positionen auf Anfang setzen
Else
  Collectedptr = Wtemp0                                     'Schreib-Position fuer Datenlaenge
End If
Icadr = Collectedptr + 2                                    'Schreib-Position fuer Daten
Collected = 0                                               'Datenmenge zurcksetzen

Enable Interrupts                                           'Interrupts aktivieren

Config Timer1 = Timer                                       'Timer1 braucht noch eine Nachbehandlung
Pllcsr = 0
Tccr1b = Measure_freq

Do
  Tcnt1 = 12                                                'Timer1 einstellen
  Tifr.tov1 = 1                                             'Timer1 Overflow Flag lschen
  Tccr1a = &B00100100                                       'LED2 einschalten
  Tccr1a = &B00110000

  Gosub Getrx0
  R = Wtemp0lo

                                                            'Blinkmuster einstellen
  Ocr1b = 32                                                'LED kurz an, lang aus
  If Wtemp0lo > Minrx Then
    Ocr1b = 223                                             'LED lang an, kurz aus
  End If
  If Icadr > Last_icadr Then                                'Speicher ist bereits voll,
    Ocr1b = 128                                             'LED mittel an, mittel aus
  End If                                                    'und weiteres Speichern verhindern


  Gosub Erfassung

  Gosub Transmit

'  T = R

                                                            'Nullpunktanzeige Hhensensor
  Set Portb.4                                               'LED3 ausschalten
  If H > 1008 Then Reset Portb.4                            'LED3 einschalten
  If H > 1040 Then Set Portb.4                              'LED3 ausschalten


  If Ocr1b = 223 Then                                      'Kanal weit genug aufgedreht?
    Wtemp0 = N                                              'dann Datensatz speichern
    Gosub Write_eeprom
    Wtemp0 = I
    Gosub Write_eeprom
    Wtemp0hi = Low(u)
    Wtemp0lo = Low(t)
    Gosub Write_eeprom
    Wtemp0 = H
    Gosub Write_eeprom
    Incr Collected
  Else
    If Collected > 0 Then
      Icadrsave = Icadr                                     'Wenn nicht Datenptr speichern
      Icadr = 4                                             'in Byte 4+5 des EEproms wird der Anfang des
      Wtemp0 = Icadrsave                                    'nchsten Datenblocks gesichert
      Gosub Write_eeprom
      Icadr = Collectedptr                                  'Schreibposition der Datenmenge laden
      Wtemp0 = Collected                                    'Datenmenge soll geschrieben werden
      Gosub Write_eeprom                                    'Und im EEPROM ablegen
      Icadr = Icadrsave                                     'Restore des Datenptr
      Collectedptr = Icadrsave                              'Naechste Schreibposition
      Wtemp0 = 0                                            ' Datenlaenge initialisieren
      Gosub Write_eeprom                                    'und speichern
      Collected = 0                                         'Datenmenge wieder auf 0 setzen
    End If
  End If

  Do                                                        'warten bis Timer1 berluft,
  Loop Until Tifr.tov1 = 1                                  'dann beginnt nchster Messzyklus

Loop

End

'#################################################################################################################################
'#
'#  "Subroutinen"
'#
'#################################################################################################################################

'-----------------------------------
'Ausgabe:
'  Icadr = 0
'  While Icadr < Eeprom_size                                 'einfach den ganzen EEProm ausgeben
'    Gosub Read_eeprom
'    Printbin #1 , Wtemp0
'  Wend
'Do
'Loop

'-----------------------------------
Transmit:
'sende "Dietrich Meissner"-Format normal seriell
  X = 0
  Printbin #1 , &HFF ; &H00 ; &H03 ; &HAA ; &HAA            'sende Header
  Printbin #1 , &H05                                        'sende Start
  Printbin #1 , &B10110010 ; &B10100010                     'sende Config
  Wtemp0lo = 84
  For Btemp = 16 To 3 Step -1                               'die Daten mssen HI-LO gesendet werden,
    Printbin #1 , Sendbuffer(btemp) ; 1                     'stehen aber LO-HI im Buffer
    Wtemp0lo = Wtemp0lo + Sendbuffer(btemp)                 'daher Ausgabe "rckwrts"
  Next
  Printbin #1 , Wtemp0lo                                    'sende Checksum

'Sende Normal Seriell
'  Printbin #1 , &B01010101 ; &B00101010
'  Printbin #1 , Sendbuffer(1) ; 9

'sende Manchester seriell
'  Printbin #1 , &B01010101 ; &B00101010
'  For Wtemp0lo = 1 To 9
'    Wtemp1 = 0
'    For Wtemp0hi = 0 To 7
'      Shift Wtemp1 , Left , 2
''      If Sendbuffer(wtemp0lo).wtemp0hi = 0 Then
''        Incr Wtemp1                                         'Wtemp1 = Wtemp1 + 1 (01)
''      Else
''        Incr Wtemp1                                         'Wtemp1 = Wtemp1 + 2 (10)
''        Incr Wtemp1
''      End If
'      Incr Wtemp1                                           'Wtemp1 = Wtemp1 + 1 (01)
'      If Sendbuffer(wtemp0lo).wtemp0hi = 1 Then
'        Incr Wtemp1                                         'Wtemp1 = Wtemp1 + 2 (10)
'      End If
'    Next
'    Printbin #1 , Wtemp1
'  Next Wtemp0lo
Return

'RC-Kanal lesen
'-----------------------------------
Getrx0:                                                     'RC-Kanal auslesen
                                                            'Rckgabe: Byte in wtemp0lo
                                                            '     0 --> pin lnger als 25ms low
                                                            '   255 --> pin lnger als 5ms high
                                                            '1..254 --> Pulsdauer:   1 .. 0.5ms oder kleiner
                                                            '                      128 .. 1.5ms
                                                            '                      254 .. 2.5ms oder grer
                                                            '                      Auflsung 8us/step
#if Use_positiv_ppm = 1
$asm
 push zl
 in   zl,sreg
 push zl
 push zh

 ldi  zl,4
 Out Tccr0 , Zl                                             'Vorteiler Timer0 auf 256

 ldi  zl,235                                                'max. 5ms warten
 Out Tcnt0 , Zl                                             'bis pin low wird (high/low-Flanke)
 in   zl,Tifr
 ori  zl,2
 Out Tifr , Zl
 ldi  zl,255
_getrx0_l1:
 in   zh,Tifr
 sbrc zh,tov0
 rjmp _getrx0_l5                                            'Zeit ist um, also ende
 sbic pinb,5                                                'pin ist noch high, also weiter warten
 rjmp _getrx0_l1

 ldi  zl,155                                                'max 25ms warten
 Out Tcnt0 , Zl                                             'bis pin high wird (low/high Flanke)
 in   zl,Tifr
 ori  zl,2
 Out Tifr , Zl
 ldi  zl,0
_getrx0_l2:
 in   zh,Tifr
 sbrc zh,Tov0
 rjmp _getrx0_l5                                            'Zeit ist um, also ende
 sbis pinb,5                                                'pin ist noch low, also weiter warten
 rjmp _getrx0_l2

 ldi  zl,2                                                  'RC-Impulsdauer messen
 Out Tccr0 , Zl                                             'Vorteiler Timer0 auf 8

 ldi  zl,190                                                'erst mal 0.5ms Pause
 Out Tcnt0 , Zl
 in   zl,Tifr
 ori  zl,2
 Out Tifr , Zl
 ldi  zl,1
_getrx0_l3:
 sbis pinb,5
 rjmp _getrx0_l5
 in   zh,Tifr
 sbrs zh,Tov0
 rjmp _getrx0_l3

 ldi  zl,1                                                  'max 2ms warten
 Out Tcnt0 , Zl                                             'bis pin low wird (high/low Flanke)
 in   zl,tifr
 ori  zl,2
 Out Tifr , Zl
 ldi  zl,254
_getrx0_l4:
 in   zh,Tifr
 sbrc zh,Tov0
 rjmp _getrx0_l5                                            'Zeit ist um, also ende
 sbic pinb,5                                                'pin ist noch high, also weiter warten
 rjmp _getrx0_l4

 in  zl,Tcnt0
_getrx0_l5:
 sts {wtemp0lo},zl

 pop zh
 pop zl
 Out Sreg , Zl
 pop zl
$end Asm
#else
#endif
Return

'Datenerfassung
'-----------------------------------
Erfassung:
      Config Timer0 = Timer , Prescale = 8
      Timer0 = 0                                            'Drehzahlmessung ber Impulsdauermessung
      Wtimer0 = 0                                           ' Genauigkeit 8us, Auflsung 13 Bit
      Tifr.tov0 = 1                                         ' Berechnung:
      Enable Ovf0                                           'Drehzahl = 7500000 / x / (Anzahl der Impulse pro Umdrehung)
      Enable Aci
      Wcompare = 65535
_getrpm_l1:                                                 ' max 65ms warten
      If Wtimer0 >= 8192 Then Goto _getrpm_l3               'ob ein Impuls kommt
      If Wcompare = 65535 Then Goto _getrpm_l1              '
      Wcompare = 65535
_getrpm_l2:
      If Wtimer0 >= 8192 Then Goto _getrpm_l3               'Zeit bis zum nchsten Impuls messen,
      If Wcompare = 65535 Then Goto _getrpm_l2              'max. 65ms
_getrpm_l3:
      Disable Aci
      Disable Ovf0
      N = Wcompare

'      I = 0                                                 'Strommessung mit internem Verstrker
'      Wtemp0 = Getadc(&B11001011)                           'REFS0 + REFS1 + (ADC +0/-1 20x) - Current
'      Waitus 150
'      For Btemp = 1 To 16
'        Enable Adc
'        Mcucr.se = 1
'        Mcucr.sm0 = 1
'        Mcucr.sm1 = 0
'        !sleep
'        Wtemp0lo = Adcl
'        Wtemp0hi = Adch
'        Disable Adc
'        I = I + Wtemp0
'      Next
'      Shift I , Right , 4

      I = 0                                                 'Strommessung mit externem Verstrker
      For Btemp = 1 To 16                                   'Mittelwertbildung aus 16 Messungen (weniger Rauschen)
        Wtemp0 = Getadc(&B11000001)                         'REFS0 + REFS1 + (ADC 1) - Current with ext amplifier
        I = I + Wtemp0
      Next
      Shift I , Right , 4                                   'entspricht /16

      U = Getadc(&B11000010)                                'REFS0 + REFS1 + ADLAR + (ADC 2) - Battery Voltage
      Shift U , Right , 2

      T = Getadc(&B11000011)                                'REFS0 + REFS1 + (ADC 3) - Motor temprature
      Shift T , Right , 2

      H = 0                                                 'Hhenmessung
      For Btemp = 1 To 16                                   'Mittelwertbildung (weniger Rauschen; bessere Auflsung?)
        Wtemp0 = Getadc(&B11000100)                         'REFS0 + REFS1 + (ADC 4) - Alti-Sensor
        H = H + Wtemp0
      Next
      Shift H , Right , 2                                   'entspricht /4, das Ergebnis hat 12Bit

'      Y = 0                                                 'Messung Versorgungsspannung Logger
'      For Btemp = 1 To 16                                   'Mittelwertbildung (weniger Rauschen)
'        Wtemp0 = Getadc(&B00011110)                         'AVCC vs Bandgap (1.18V)
'        Y = Y + Wtemp0
'      Next
'      Shift Y , Right , 6                                   'entspricht /64, das Ergebnis hat 8 Bit
Return

'Interrupt-Routinen
'-----------------------------------
_adc:
 Return

_ovf0:
$asm
 push zl                                                    '
 in zl,SREG                                                 '
 push zl                                                    '

 lds zl,{Wtimer0hi}                                         ' WTimer0 bildet einen 16Bit-Timer
 inc zl                                                     ' High-Byte von WTimer0 wird
 sts {Wtimer0hi},zl                                         ' hochgezhlt

 pop zl                                                     '
 Out Sreg , Zl                                              '
 pop zl                                                     '
$end Asm

' Incr Wtimer0hi                                            '

Return

_aci:
$asm
 push zl                                                    '
 in zl,SREG                                                 '
 push zl                                                    '

 in  zl,tcnt0                                               '
 sts {wtimer0lo},zl                                         '
 ldi zl,0                                                   '
 Out Tcnt0 , Zl                                             '
 lds zl,{wtimer0lo}                                         '
 sts {wcomparelo},zl                                        '
 lds zl,{wtimer0hi}                                         '
 sts {wcomparehi},zl                                        '
 ldi zl,0                                                   '
 sts {wtimer0hi} , Zl                                       '

 pop zl                                                     '
 Out Sreg , Zl                                              '
 pop zl                                                     '
$end Asm

' Wtimer0lo = Timer0                                        ' Timer0 und WTimer0
' Timer0 = 0                                                ' werden bei Comparator Int
' Wcompare = Wtimer0                                        ' zurckgesetzt
' Wtimer0hi = 0                                             '

Return

'I2C-Routinen
'-----------------------------------
Write_eeprom:
    I2cstart                                                'start condition
    I2cwbyte Eeprom_adress_w                                'slave address
    I2cwbyte Icadrhi                                        'address of EEPROM
    I2cwbyte Icadrlo                                        'address of EEPROM
    I2cwbyte Wtemp0hi                                       'value to write
    I2cwbyte Wtemp0lo                                       'value to write
    I2cstop                                                 'stop condition
    Waitms 10                                               'wait for 10 milliseconds
    Incr Icadr
    Incr Icadr
Return

Read_eeprom:
    I2cstart                                                'generate start
    I2cwbyte Eeprom_adress_w                                'slave adsress
    I2cwbyte Icadrhi                                        'address of EEPROM
    I2cwbyte Icadrlo                                        'address of EEPROM
    I2cstart                                                'repeated start
    I2cwbyte Eeprom_adress_r                                'slave address (read)
    I2crbyte Wtemp0hi , Ack                                 'read byte
    I2crbyte Wtemp0lo , Nack                                'read byte
    I2cstop                                                 'generate stop
    Incr Icadr
    Incr Icadr
Return

'AVR-internes EEProm
'-----------------------------------
$eeprom
Data 0                                                      'Byte0  : Dummy
Data 4095%                                                  'Byte1+2: Eeprom_size
                                                             '         65535 (FF FF) - 24c512 (64kByte)
                                                             '         32767 (7F FF) - 24c256 (32kByte)
                                                             '         16383 (3F FF) - 24c128 (16kByte)
                                                             '          8191 (1F FF) - 24c64  (8kByte)
                                                             '          4095 (0F FF) - 24c32  (4kByte)
Data 8                                                      'Byte3  : Measured_bytes
Data 4083%                                                  'Byte4+5: Last_icadr = Eeprom_size - Measured_bytes - 4
Data 90                                                     'Byte6  : Minrx
Data 13                                                     'Byte7  : Measure_freq
                                                             '         15 - berlauf nach 4,192 Sekunden bei 1MHz
                                                             '         14 - berlauf nach 2,096 Sekunden bei 1MHz
                                                             '         13 - berlauf nach 1,048 Sekunden bei 1MHz
                                                             '         12 - berlauf nach 0,524 Sekunden bei 1MHz
                                                             '         11 - berlauf nach 0,262 Sekunden bei 1MHz

'
'Datenformat im I2C-EEProm
'-----------------------------------
'
'Datensatz: Ergebnis eines Messdurchgangs, z.B. Drehzahl, Strom, Spannung, Temperatur
'           Lnge eines Datensatzes = Measured_bytes
'
'Datenblock: Anzahl von Datenstzen
'
'
'
'Header ------------------------- 4 Byte:   Byte0    Version        (Versionsnummer)
'                                           Byte1    Measured_bytes (Anzahl der Bytes pro Datensatz)
'                                           Byte2+3  EEProm_size    (I2C-EEPromgrsse Div 256)
'Anfang des letzten Datenblocks - 2 Byte:   Byte4+5
'Anzahl der Datenstze
'  im nchsten Datenblock ------- 2 Byte:   Byte6+7
'Datenstze
'   :
'   :
'   :
'Anzahl der Datenstze
'  im nchsten Datenblock ------- 2 Byte:
'
'Bsp. in ASCII:
'15;           Version=1.5
'6;            Measured_bytes=6
'0;63;         EEProm_size=16384(63*256+255)
'0;58;         Anfang des letzten Datenblocks, hier wird die nchste Aufzeichnung begonnen
'0;5;          im nchsten Block sind 5 Datenstze
'0;0;0;0;0;0;
'0;0;0;0;0;0;
'0;0;0;0;0;0;
'0;0;0;0;0;0;
'0;0;0;0;0;0;
'0;3;          im nchsten Block sind 3 Datenstze
'0;0;0;0;0;0;
'0;0;0;0;0;0;
'0;0;0;0;0;0;
'0;0;          im nchsten Block sind 0 Datenstze = Ende der Aufzeichnung